/*
  Copyright (C) 2004  Jonathan Harrington jonathan@skynet

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
package sync4j.source;

import java.io.File;
import java.io.IOException;
import java.io.FileNotFoundException;
import java.io.Serializable;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.util.Properties;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.Iterator;
import java.util.logging.Logger;
import java.util.logging.Level;
import java.security.Principal;
import java.sql.Timestamp;

import sync4j.framework.logging.Sync4jLogger;
import sync4j.framework.engine.SyncItemImpl;
import sync4j.framework.engine.SyncItem;
import sync4j.framework.engine.SyncItemKey;
import sync4j.framework.engine.SyncProperty;
import sync4j.framework.engine.SyncItemState;
import sync4j.framework.engine.source.SyncSource;
import sync4j.framework.engine.source.SyncSourceException;
import sync4j.framework.engine.source.AbstractSyncSource;
import sync4j.framework.tools.Base64;
import sync4j.framework.tools.IOTools;

import javax.mail.*;
import javax.mail.internet.*;
import java.io.IOException;
import java.io.NotSerializableException;

import javax.naming.InitialContext;
import javax.naming.Context;
import javax.naming.NamingException;
import javax.rmi.PortableRemoteObject;
import java.rmi.RemoteException;
import javax.ejb.CreateException;
import java.util.Properties;
import java.util.ArrayList;

import ie.skynet.jonathan.groupware.GwException;
import ie.skynet.jonathan.groupware.GwConnection;
import ie.skynet.jonathan.groupware.ContactManager;
import ie.skynet.jonathan.groupware.TaskManager;
import ie.skynet.jonathan.groupware.AppointmentManager;
import ie.skynet.jonathan.groupware.Contact;
import ie.skynet.jonathan.groupware.Appointment;
import ie.skynet.jonathan.groupware.Task;

import java.io.BufferedReader;
import java.io.FileReader;
import java.io.FileWriter;

import java.sql.*;
import java.util.regex.Pattern;

	public abstract class SyncSourceBase extends AbstractSyncSource implements SyncSourceBaseInterface
	{
	// ------------------------------------------------------------ Constructors

	public SyncSourceBase() {}

	public SyncSourceBase(String name) {
		super(name);
	}

	// ---------------------------------------------------------- public Methods

	public String getFile(Principal principal, String sourceURI, String fileName) throws SyncSourceException
	{
		try{
			GwConnection gwConnection = new GwConnection(getHost(), getUsername(principal), getPassword(principal));

			String file = null;

			if(sourceURI.equals(getContactFolderName()))
			{
				file = this.getContact(gwConnection, fileName);
			}
			if(sourceURI.equals(getEventFolderName()))
			{
				file = this.getAppointment(gwConnection, fileName);
			}
			if(sourceURI.equals(getTodoFolderName()))
			{
				file = this.getTask(gwConnection, fileName);
			}

			return file;
		} catch (GwException e) { throw new SyncSourceException(e.getMessage(), e); }
	}

	public void deleteFile(Principal principal, String sourceURI, String fileName) throws SyncSourceException
{
		try{
			GwConnection gwConnection = new GwConnection(getHost(), getUsername(principal), getPassword(principal));

			if(sourceURI.equals(getContactFolderName()))
			{
				this.deleteContact(gwConnection, fileName);
			}
			if(sourceURI.equals(getEventFolderName()))
			{
				this.deleteAppointment(gwConnection, fileName);
			}
			if(sourceURI.equals(getTodoFolderName()))
			{
				this.deleteTask(gwConnection, fileName);
			}

		} catch (GwException e) { throw new SyncSourceException(e.getMessage(), e); }
	}

	public String[] getFiles(Principal principal, String sourceURI) throws SyncSourceException
	{
		try {
			System.out.println(getHost()+" "+getUsername(principal)+" "+getPassword(principal));
			GwConnection gwConnection = new GwConnection(getHost(), getUsername(principal), getPassword(principal));

			String[] files = null;
			// Find out what we are Syncing. The SourceURI contains the name passed by the client so
			// we can use this. We then call the needed functions.
			if(sourceURI.equals(getContactFolderName()))
			{
				files = this.getContacts(gwConnection);
			}
			if(sourceURI.equals(getEventFolderName()))
			{
				files = this.getAppointments(gwConnection);
			}
			if(sourceURI.equals(getTodoFolderName()))
			{
				files = this.getTasks(gwConnection);
			}
			return files;
		} catch (GwException e) { System.out.println("in getFiles throw"); throw new SyncSourceException(e.getMessage(), e); }
	}

	public String[] getFileNames(Principal principal, String sourceURI) throws SyncSourceException
	{
		try {
			GwConnection gwConnection = new GwConnection(getHost(), getUsername(principal), getPassword(principal));

			String[] fileNames = null;
			// Find out what we are Syncing. The SourceURI contains the name passed by the client so
			// we can use this. We then call the needed functions.
			if(sourceURI.equals(getContactFolderName()))
			{
				fileNames = this.getContactNames(gwConnection);
			}
			if(sourceURI.equals(getEventFolderName()))
			{
				fileNames = this.getAppointmentNames(gwConnection);
			}
			if(sourceURI.equals(getTodoFolderName()))
			{
				fileNames = this.getTaskNames(gwConnection);
			}
			return fileNames;
		} catch (GwException e) { throw new SyncSourceException(e.getMessage(), e); }
	}

	public long getLastModified(Principal principal, String sourceURI, String fileName) throws SyncSourceException
	{
		try {
			GwConnection gwConnection = new GwConnection(getHost(), getUsername(principal), getPassword(principal));

			long lastModified = 0;

			if(sourceURI.equals(getContactFolderName()))
			{
				lastModified = getLastModifiedContact(gwConnection, fileName);
			}
			if(sourceURI.equals(getEventFolderName()))
			{
				lastModified = getLastModifiedAppointment(gwConnection, fileName);
			}
			if(sourceURI.equals(getTodoFolderName()))
			{
				lastModified = getLastModifiedTask(gwConnection, fileName);
			}
			return lastModified;
		} catch (GwException e) { throw new SyncSourceException(e.getMessage(), e); }
	}

	public void addFile(Principal principal, String sourceURI, byte[] fileContent, String fileName) throws SyncSourceException
	{
		try {
			GwConnection gwConnection = new GwConnection(getHost(), getUsername(principal), getPassword(principal));

			// Find out what we are Syncing. The SourceURI contains the name passed by the client so
			// we can use this. We then call the apropriate function
			if(sourceURI.equals(getContactFolderName()))
			{
				this.addContact(principal, gwConnection, fileContent, fileName);
			}
			if(sourceURI.equals(getEventFolderName()))
			{
				this.addAppointment(principal, gwConnection, fileContent, fileName);
			}
			if(sourceURI.equals(getTodoFolderName()))
			{
				this.addTask(principal, gwConnection, fileContent, fileName);
			}
		} catch (GwException e) { throw new SyncSourceException(e.getMessage(), e); }
	}

	public void deleteFiles(Principal principal, String getSourceURI) throws SyncSourceException
	{
		try {
			GwConnection gwConnection = new GwConnection(getHost(), getUsername(principal), getPassword(principal));

			// Find out what we are Syncing. The SourceURI contains the name passed by the client so
			// we can use this. We then call the apropriate function
			if(sourceURI.equals(getContactFolderName()))
			{
				this.deleteContacts(principal, gwConnection);
			}
			if(sourceURI.equals(getEventFolderName()))
			{
				this.deleteAppointments(principal, gwConnection);
			}
			if(sourceURI.equals(getTodoFolderName()))
			{
				this.deleteTasks(principal, gwConnection);
			}
		} catch (GwException e) { throw new SyncSourceException(e.getMessage(), e); }
	}

	// ---------------------------------------------------------- private Methods
	// ------------------------------------------ Delete all data methods

	public void deleteContacts(Principal principal, GwConnection gwConnection) throws GwException
	{
		ContactManager fileManager = new ContactManager(gwConnection, getGwServerContactFolderName());
		fileManager.deleteFiles();
	}

	public void deleteAppointments(Principal principal, GwConnection gwConnection) throws GwException
	{
		AppointmentManager fileManager = new AppointmentManager(gwConnection, getGwServerEventFolderName());
		fileManager.deleteFiles();
	}

	public void deleteTasks(Principal principal, GwConnection gwConnection) throws GwException
	{
		TaskManager fileManager = new TaskManager(gwConnection, getGwServerTaskFolderName());
		fileManager.deleteFiles();
	}

	// ------------------------------------------ Retreive data methods
	private String[] getContacts(GwConnection gwConnection) throws GwException
	{
		ContactManager fileManager = new ContactManager(gwConnection, getGwServerContactFolderName());
		return fileManager.getFiles();
	}

	private String getContact(GwConnection gwConnection, String fileName) throws GwException
	{
		ContactManager fileManager = new ContactManager(gwConnection, getGwServerContactFolderName());
		return fileManager.getFile(fileName);
	}

	private String[] getContactNames(GwConnection gwConnection) throws GwException
	{
		ContactManager fileManager = new ContactManager(gwConnection, getGwServerContactFolderName());
		return fileManager.getFileNames();
	}

	private String[] getAppointments(GwConnection gwConnection) throws GwException
	{
		AppointmentManager fileManager = new AppointmentManager(gwConnection, getGwServerEventFolderName());
		return fileManager.getFiles();
	}

	private String getAppointment(GwConnection gwConnection, String fileName) throws GwException
	{
		AppointmentManager fileManager = new AppointmentManager(gwConnection, getGwServerEventFolderName());
		return fileManager.getFile(fileName);
	}

	private String[] getAppointmentNames(GwConnection gwConnection) throws GwException
	{
		AppointmentManager fileManager = new AppointmentManager(gwConnection, getGwServerEventFolderName());
		return fileManager.getFileNames();
	}

	private String[] getTasks(GwConnection gwConnection) throws GwException
	{
		//FIXME URGENCY TOP why is this breaking things.
		TaskManager fileManager = new TaskManager(gwConnection); //, getGwServerTaskFolderName());
		return fileManager.getFiles();
	}

	private String getTask(GwConnection gwConnection, String fileName) throws GwException
	{
		TaskManager fileManager = new TaskManager(gwConnection, getGwServerTaskFolderName());
		return fileManager.getFile(fileName);
	}

	private String[] getTaskNames(GwConnection gwConnection) throws GwException
	{
		TaskManager fileManager = new TaskManager(gwConnection, getGwServerTaskFolderName());
		return fileManager.getFileNames();
	}

	private void deleteContact(GwConnection gwConnection, String fileName) throws GwException
	{
		ContactManager fileManager = new ContactManager(gwConnection, getGwServerContactFolderName());
		fileManager.deleteFile(fileName);
	}

	private void deleteAppointment(GwConnection gwConnection, String fileName) throws GwException
	{
		AppointmentManager fileManager = new AppointmentManager(gwConnection, getGwServerEventFolderName());
		fileManager.deleteFile(fileName);
	}

	private void deleteTask(GwConnection gwConnection, String fileName) throws GwException
	{
		TaskManager fileManager = new TaskManager(gwConnection, getGwServerTaskFolderName());
		fileManager.deleteFile(fileName);
	}

	private long getLastModifiedContact(GwConnection gwConnection, String fileName) throws GwException
	{
		ContactManager fileManager = new ContactManager(gwConnection, getGwServerContactFolderName());
		return fileManager.getLastModified(fileName);
	}

	private long getLastModifiedAppointment(GwConnection gwConnection, String fileName) throws GwException
	{
		AppointmentManager fileManager = new AppointmentManager(gwConnection, getGwServerEventFolderName());
		return fileManager.getLastModified(fileName);
	}

	private long getLastModifiedTask(GwConnection gwConnection, String fileName) throws GwException
	{
		TaskManager fileManager = new TaskManager(gwConnection, getGwServerTaskFolderName());
		return fileManager.getLastModified(fileName);
	}


	// ------------------------------------------ Add data methods
	private void addContact(Principal principal, GwConnection gwConnection, byte[] fileContent, String fileName) throws GwException
	{
		try {
			ContactManager fileManager = new ContactManager(gwConnection, getGwServerContactFolderName());

			File f = new File(fileName);
			IOTools.writeFile(fileContent, f);
			BufferedReader bufferedReader = new BufferedReader(new FileReader(fileName));

			Contact file = new Contact(bufferedReader);
			fileManager.addFile(file.getString(), fileName);

			// Delete the temporary file we needed
			new File(fileName).delete();
		} catch (IOException e ) { throw new GwException(e.getMessage(), e);}
	}

	private void addTask(Principal principal, GwConnection gwConnection, byte[] fileContent, String fileName) throws GwException
	{
		try {
			TaskManager fileManager = new TaskManager(gwConnection, getGwServerTaskFolderName());

			File f = new File(fileName);
			IOTools.writeFile(fileContent, f);
			BufferedReader bufferedReader = new BufferedReader(new FileReader(fileName));

			Task file = new Task(bufferedReader);
			fileManager.addFile(file.getString(), fileName);

			// Delete the temporary file we needed
			new File(fileName).delete();
		} catch (IOException e ) { throw new GwException(e.getMessage(), e);}
	}

	private void addAppointment(Principal principal, GwConnection gwConnection, byte[] fileContent, String fileName) throws GwException
	{
		try {
			AppointmentManager fileManager = new AppointmentManager(gwConnection, getGwServerEventFolderName());

			File f = new File(fileName);
			IOTools.writeFile(fileContent, f);
			BufferedReader bufferedReader = new BufferedReader(new FileReader(fileName));

			Appointment file = new Appointment(bufferedReader);
			fileManager.addFile(file.getString(), fileName);

			// Delete the temporary file we needed
			new File(fileName).delete();
		} catch (IOException e ) { throw new GwException(e.getMessage(), e);}
	}

	// ------------------------------------------ Database methods
		public String getHost() throws SyncSourceException
		{
			String host = null;

			try {
				Class.forName(getJdbcDriver());
				Connection con = DriverManager.getConnection(getJdbcUrl(), "sa", "");


				String sSQL = "Select gw_hostname from sync4j_groupware_host";
				Statement stmt = con.createStatement();
				ResultSet rs = stmt.executeQuery(sSQL);

				while(rs.next())
				{
					host = rs.getString(1);
				}
			} catch (ClassNotFoundException e) { throw new SyncSourceException("Unable to load JDBC driver", e);
			} catch (SQLException e) { throw new SyncSourceException("Unable to retreive host", e); }

			return host;
		}

		public String getUsername(Principal principal) throws SyncSourceException
		{
			String username = null;
			try {
				Class.forName(getJdbcDriver());
				Connection con = DriverManager.getConnection(getJdbcUrl(), "sa", "");


				String sSQL = "Select gw_username from sync4j_groupware_users where (gw_principal='"+principal.toString()+"')";
				Statement stmt = con.createStatement();
				ResultSet rs = stmt.executeQuery(sSQL);

				while(rs.next())
				{
					username = rs.getString(1);
				}
			} catch (ClassNotFoundException e) { throw new SyncSourceException("Unable to load JDBC driver", e);
			} catch (SQLException e) { throw new SyncSourceException("Unable to retreive password", e); }

			//return username;
			return "harry";
		}

		public String getPassword(Principal principal) throws SyncSourceException
		{
			String uName = getUsername(principal);
			String password = null;

			try {
				Class.forName(getJdbcDriver());
				Connection con = DriverManager.getConnection(getJdbcUrl(), "sa", "");


				String sSQL = "Select gw_password from sync4j_groupware_users where (gw_username='"+getUsername(principal)+"')";
				Statement stmt = con.createStatement();
				ResultSet rs = stmt.executeQuery(sSQL);

				while(rs.next())
				{
					password = rs.getString(1);
				}
			} catch (ClassNotFoundException e) { throw new SyncSourceException("Unable to load JDBC driver", e);
			} catch (SQLException e) { throw new SyncSourceException("Unable to retreive password", e); }

			return password;
		}

		public String getTodoFolderName() {
			return "gwtasks";
		}

		public String getContactFolderName() {
			return "gwcontact";
		}

		public String getEventFolderName() {
			return "gwcalendar";
		}

		public String getGwServerEventFolderName() throws GwException
		{
			String folder = null;

			try {
				Class.forName(getJdbcDriver());
				Connection con = DriverManager.getConnection(getJdbcUrl(), "sa", "");


				String sSQL = "Select gw_calendar from sync4j_groupware_folders";
				Statement stmt = con.createStatement();
				ResultSet rs = stmt.executeQuery(sSQL);

				while(rs.next())
				{
					folder = rs.getString(1);
				}
			} catch (ClassNotFoundException e) { throw new GwException("Unable to load JDBC driver", e);
			} catch (SQLException e) { throw new GwException("Unable to retreive Event Foldername", e); }

			return folder;
		}

		public String getGwServerContactFolderName() throws GwException
		{
			String folder = null;

			try {
				Class.forName(getJdbcDriver());
				Connection con = DriverManager.getConnection(getJdbcUrl(), "sa", "");


				String sSQL = "Select gw_contact from sync4j_groupware_folders";
				Statement stmt = con.createStatement();
				ResultSet rs = stmt.executeQuery(sSQL);

				while(rs.next())
				{
					folder = rs.getString(1);
				}
			} catch (ClassNotFoundException e) { throw new GwException("Unable to load JDBC driver", e);
			} catch (SQLException e) { throw new GwException("Unable to retreive ContactFoldername", e); }

			return folder;
		}

		public String getGwServerTaskFolderName() throws GwException
		{
			String folder = null;

			try {
				Class.forName(getJdbcDriver());
				Connection con = DriverManager.getConnection(getJdbcUrl(), "sa", "");


				String sSQL = "Select gw_task from sync4j_groupware_folders";
				Statement stmt = con.createStatement();
				ResultSet rs = stmt.executeQuery(sSQL);

				while(rs.next())
				{
					folder = rs.getString(1);
				}
			} catch (ClassNotFoundException e) { throw new GwException("Unable to load JDBC driver", e);
			} catch (SQLException e) { throw new GwException("Unable to retreive Task folder", e); }

			return folder;
		}


	// Overriden by parent
	public String getJdbcDriver()
	{
		return null;
	}

	public String getJdbcUrl()
	{
		return null;
	}

	// ------------------------------------------ Not used in this release
	private void setDbState(Principal principal, String fileName,char state, Timestamp t) throws SyncSourceException
	{
		System.out.println("In setState\n");
		try {
			Class.forName(getJdbcDriver());
			Connection con = DriverManager.getConnection(getJdbcUrl(), "sa", "");

			// FIXME URGENCY top should change depending on sourceURI
			String tableName = "sync4j_groupware_gwcontact";

			String sSQL = "Select gw_state from "+tableName+" where (gw_principal='"+principal.toString()+"' AND gw_key='"+fileName+"')";
			System.out.println(sSQL);
			Statement stmt = con.createStatement();
			ResultSet rs = stmt.executeQuery(sSQL);

			boolean there;
			try {
				there = rs.first();
				there = true;
			} catch (SQLException e) {
				there = false;
			}
			stmt.close();

			System.out.println("There is " +there);

			// If value not already in table Insert it,
			// else update it.
			if(!there)
			{
				Statement stmt1 = con.createStatement();
				sSQL = "insert into "+tableName+" (gw_principal, gw_key, gw_state, gw_timestamp) values ('"+principal.toString()+"', '"+fileName+"', '"+state+"', '"+t+"')";
				System.out.println(sSQL);
				//ResultSet rs1 = stmt1.executeQuery(sSQL);
				int iTotalUpdated = stmt1.executeUpdate(sSQL);
				stmt1.close();
			} else {
				Statement stmt2 = con.createStatement();
				sSQL = "update "+tableName+" set (gw_state='"+state+"', gw_timestamp='"+t+"') where (gw_principal='"+principal.toString()+"' AND gw_key='"+fileName+"')";
				System.out.println(sSQL);
				int iTotalUpdated = stmt2.executeUpdate(sSQL);
				stmt2.close();
			}

			con.close();


		} catch (ClassNotFoundException e) {throw new SyncSourceException("Error loading jdbc driver", e);
		} catch (SQLException e){ System.out.println("Error connecting to Database "+ e);
		throw new SyncSourceException("Error connecting to Database", e);
		}
	}
}
