/*
  Copyright (C) 2004  Jonathan Harrington jonathan@skynet

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
package sync4j.source;

import sync4j.framework.engine.source.SyncSourceException;
import java.security.Principal;

public interface SyncSourceBaseInterface
{
	/**
	* Returns a file from the data store
	*
	*/
	public String getFile(Principal principal, String sourceURI, String fileName) throws SyncSourceException;

	/**
	* Deletes a file from the data store
	*
	*/
	public void deleteFile(Principal principal, String sourceURI, String fileName) throws SyncSourceException;

	/**
	* Returns all the files in the data store
	*
	*/
	public String[] getFiles(Principal principal, String sourceURI) throws SyncSourceException;

	/**
	* Returns all the names of the files in the data store
	*
	*/
	public String[] getFileNames(Principal principal, String sourceURI) throws SyncSourceException;

	/**
	* Returns a timestamp containing the lastmodified value of a file from the data store
	*
	*/
	public long getLastModified(Principal principal, String sourceURI, String fileName) throws SyncSourceException;

	/**
	* Adds a file to the data store
	*
	*/
	public void addFile(Principal principal, String sourceURI, byte[] fileContent, String fileName) throws SyncSourceException;
}
